function Qlty_3=IndSolSelUnTxUnSx(CsolTx,CsolSx,Qsub,K,gammaSol,M,N,NumTx,NumSx)
% Qlty_3=IndSolSelUnTxUnSx(CsolTx,CsolSx,Qsub,K,gammaSol,M,N,NumTx,NumSx)
%
% This routine selects the individual minimal measurement solutions
% by unknown transmitters and scatterers for BLUE to obtain the final solution.
%
% Input   
% CsolTx:   measurement combination for unknown transmitter solutions
% CsolSx:   measurement combination for unknown scatterer solutions
% Qsub:     measurement noise covariance matrix
% gammaSol: unknown transmitter and scatterer solutions by their measurements
% M:        number of unknown transmitters
% N:        number of unknown scatterers
% NumTx:    total number of transmitters
% NumSx:    total number of scatterers
%
% Output   
% Qlty_3:   quality factor for the solutions 
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
       
Tx_est=reshape(gammaSol(1:K*M),K,M);
Sx_est=reshape(gammaSol(K*M+1:end),K,N);

for i=1:NumTx
    for j=1:NumSx
        Txi=Tx_est(:,CsolTx(i,:));
        Sxj=Sx_est(:,CsolSx(j,:));
        
        alpha=[atan2(Txi(2,1),Txi(1,1)),atan2(Txi(2,2),Txi(1,2))];
        beta=[atan2(Sxj(2,1),Sxj(1,1)),atan2(Sxj(2,2),Sxj(1,2))];
        if K==3
            thetaSx=[asin(Sxj(3,1)/norm(Sxj(:,1))),asin(Sxj(3,2)/norm(Sxj(:,2)))];
            thetaTx=[asin(Txi(3,1)/norm(Txi(:,1))),asin(Txi(3,2)/norm(Txi(:,2)))];
        end
        
        rho_d=[((Txi(:,1)-Sxj(:,1))/norm(Txi(:,1)-Sxj(:,1))-Txi(:,1)/norm(Txi(:,1)))',zeros(1,K),((Sxj(:,1)-Txi(:,1))/norm(Txi(:,1)-Sxj(:,1))+Sxj(:,1)/norm(Sxj(:,1)))',zeros(1,K);
            ((Txi(:,1)-Sxj(:,2))/norm(Txi(:,1)-Sxj(:,2))-Txi(:,1)/norm(Txi(:,1)))',zeros(1,K),zeros(1,K),((Sxj(:,2)-Txi(:,1))/norm(Txi(:,1)-Sxj(:,2))+Sxj(:,2)/norm(Sxj(:,2)))';
            zeros(1,K),((Txi(:,2)-Sxj(:,1))/norm(Txi(:,2)-Sxj(:,1))-Txi(:,2)/norm(Txi(:,2)))',((Sxj(:,1)-Txi(:,2))/norm(Txi(:,2)-Sxj(:,1))+Sxj(:,1)/norm(Sxj(:,1)))',zeros(1,K);
            zeros(1,K),((Txi(:,2)-Sxj(:,2))/norm(Txi(:,2)-Sxj(:,2))-Txi(:,2)/norm(Txi(:,2)))',zeros(1,K),((Sxj(:,2)-Txi(:,2))/norm(Txi(:,2)-Sxj(:,2))+Sxj(:,2)/norm(Sxj(:,2)))'];
        
        rho_angleTx=[[-Txi(2,1),Txi(1,1),zeros(1,K-2)]./(norm([Txi(2,1),Txi(1,1)])^2),zeros(1,K),zeros(1,2*K);
            zeros(1,K),[-Txi(2,2),Txi(1,2),zeros(1,K-2)]./(norm([Txi(2,2),Txi(1,2)])^2),zeros(1,2*K)];
        rho_angleSx=[zeros(1,2*K),[-Sxj(2,1),Sxj(1,1),zeros(1,K-2)]./(norm([Sxj(2,1),Sxj(1,1)])^2),zeros(1,K);
            zeros(1,2*K),zeros(1,K),[-Sxj(2,2),Sxj(1,2),zeros(1,K-2)]./(norm([Sxj(2,2),Sxj(1,2)])^2)];
        if K==3
            rho_angleTx2=[[-sin(thetaTx(1))*cos(alpha(1)),-sin(thetaTx(1))*sin(alpha(1)),cos(thetaTx(1))]/norm(Txi(:,1)),zeros(1,K),zeros(1,2*K);
                zeros(1,K),[-sin(thetaTx(2))*cos(alpha(2)),-sin(thetaTx(2))*sin(alpha(2)),cos(thetaTx(2))]/norm(Txi(:,2)),zeros(1,2*K)];
            rho_angleSx2=[zeros(1,2*K),[-sin(thetaSx(1))*cos(beta(1)),-sin(thetaSx(1))*sin(beta(1)),cos(thetaSx(1))]/norm(Sxj(:,1)),zeros(1,K);
                zeros(1,2*K),zeros(1,K),[-sin(thetaSx(2))*cos(beta(2)),-sin(thetaSx(2))*sin(beta(2)),cos(thetaSx(2))]/norm(Sxj(:,2))];
            
            rho_angleTx=[rho_angleTx;rho_angleTx2];
            rho_angleSx=[rho_angleSx;rho_angleSx2];
        end
        
        B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
        CovInd=inv(B_rhoi)*Qsub*inv(B_rhoi)';
        % FIMInd=B_rhoi'*inv(Qsub)*B_rhoi;
        Qlty_3((i-1)*NumSx+j)=trace(CovInd);
    end
    
end









